import com.android.build.gradle.LibraryExtension
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.artifacts.VersionCatalogsExtension
import org.gradle.kotlin.dsl.configure
import org.gradle.kotlin.dsl.dependencies
import org.gradle.kotlin.dsl.getByType
import org.gradle.kotlin.dsl.kotlin

/**
 * 안드로이드 의존 플러그인
 *
 * @author bmo
 * @since 2023-03-10
 */
class AndroidFeatureConventionPlugin : Plugin<Project> {
    override fun apply(target: Project) {
        with(target) {
            pluginManager.apply {
                apply("kr.swkang.playground.android.library")
                apply("kr.swkang.playground.android.hilt")
            }
            extensions.configure<LibraryExtension> {
                defaultConfig {
                    testInstrumentationRunner =
                        "kr.swkang.playground.test.PlayGroundTestRunner" // "androidx.test.runner.AndroidJUnitRunner"
                }
            }

            val libs = extensions.getByType<VersionCatalogsExtension>().named("libs")
            dependencies {
                add("implementation", project(":core:data"))
                add("implementation", project(":core:common"))
                add("implementation", project(":core:domain"))
                add("implementation", project(":core:design"))
                add("implementation", project(":core:annotation"))

                // 그 외 공통 의존 들
                add("implementation", libs.findLibrary("coil.kt").get())
                add("implementation", libs.findLibrary("coil.kt.compose").get())
                add("implementation", libs.findLibrary("androidx.hilt.navigation.compose").get())
                add("implementation", libs.findLibrary("androidx.lifecycle.runtimeCompose").get())
                add("implementation", libs.findLibrary("androidx.lifecycle.viewModelCompose").get())
                add("implementation", libs.findLibrary("androidx.paging.runtime").get())
                add("implementation", libs.findLibrary("androidx.paging.compose").get())
                add("implementation", libs.findLibrary("androidx.paging.testing").get())
                add("implementation", libs.findLibrary("kotlinx.coroutines.android").get())

                // 테스트 모듈
                add("testImplementation", kotlin("test"))
                add("testImplementation", project(":core:test"))
                add("androidTestImplementation", kotlin("test"))
                add("androidTestImplementation", project(":core:test"))
            }
        }
    }
}
